#!/usr/bin/env python

"""
Acorn Electron ULA simulation controller.

Copyright (C) 2011, 2012, 2013, 2014 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from ula import *
import pygame

WIDTH = 640
HEIGHT = 512
INTENSITY = 255

def update_screen(screen, screen_array):

    """
    Update the host's 'screen' surface with the contents of the 'screen_array'
    containing the pixel content of the screen.
    """

    surface = pygame.Surface((MAX_WIDTH, MAX_HEIGHT), 0, screen)
    sa = pygame.surfarray.pixels3d(surface)
    a = sa.transpose(1, 0, 2)

    # Copy the array to a surface and apply a pixel intensity.

    try:
        a.flat[:] = screen_array
        a *= INTENSITY
    finally:
        del a, sa

    # Scale the surface to the dimensions of the host's screen and copy the
    # result to the host's screen.

    screen.blit(pygame.transform.scale(surface, (WIDTH, HEIGHT)), (0, 0))

def mainloop():
    while 1:
        pygame.display.flip()
        event = pygame.event.wait()
        if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
            break

if __name__ == "__main__":
    pygame.init()
    screen = pygame.display.set_mode((WIDTH, HEIGHT), 0)

    ula = get_ula()

    # Test MODE 2.

    ula.set_mode(2); ula.reset()

    ula.ram.fill(0x3000, 0x5800 - 320, encode((1, 6), 4))
    ula.ram.fill(0x5800 - 320, 0x8000, encode((2, 7), 4))
    ula_screen = update(ula)
    update_screen(screen, ula_screen)
    print "Screen updated."
    mainloop()

    ula.screen_start = 0x3000 + 2
    ula_screen = update(ula)
    update_screen(screen, ula_screen)
    print "Screen updated."
    mainloop()

    # Test MODE 6.

    ula.set_mode(6); ula.reset()

    ula.ram.fill(0x6000, 0x6f00 + 160, encode((1, 0, 1, 1, 0, 0, 1, 1), 1))
    ula.ram.fill(0x6f00 + 160, 0x7f40, encode((1, 0, 1, 0, 1, 0, 1, 0), 1))
    ula_screen = update(ula)
    update_screen(screen, ula_screen)
    print "Screen updated."
    mainloop()

    ula.screen_start = 0x6f00 + 160
    ula_screen = update(ula)
    update_screen(screen, ula_screen)
    print "Screen updated."
    mainloop()

# vim: tabstop=4 expandtab shiftwidth=4
